<?php

/**
 * Redirect block
 *
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_Block_Product_View extends Glopal_Connector_Block_Template
{
    public function getProductId()
    {
        return $this->getProduct()->getId();
    }

    public function getStoreUrl()
    {
        return Mage::helper('glopal_connector')->getStoreBaseUrl();
    }

    public function getLocaleCode()
    {
        return Mage::getStoreConfig('general/locale/code');
    }

    /**
     * Retrieve current product model
     *
     * @return Mage_Catalog_Model_Product
     */
    public function getProduct()
    {
        return Mage::registry('current_product');
    }

    /**
     * Get product metadata in JSON-LD format
     *
     * @return string
     */
    public function getProductJsonLd()
    {
        $product = $this->getProduct();
        $outputHelper = $this->helper('catalog/output');

        if ($product->getStockItem()->getIsInStock() >= 1) {
            $stockMessage = 'http://schema.org/InStock';
        } else {
            $stockMessage = 'http://schema.org/OutOfStock';
        }

        $json = array(
            '@context' => 'http://schema.org/',
            '@type' => 'Product',
            'name' => $outputHelper->productAttribute($product, $product->getName(), 'name'),
            'image' => $product->getImageUrl(),
            'description' => $outputHelper->productAttribute($product, nl2br($product->getDescription()), 'description'),
            'sku' => $product->getSku(),
            'offers' => array(
                '@type' => 'Offer',
                'url' => $product->getProductUrl(),
                'priceCurrency' => Mage::app()->getStore()->getCurrentCurrencyCode(),
                'price' => number_format($product->getFinalPrice(), 2, '.', ''),
                'availability' => $stockMessage
            )
        );

        return json_encode($json, JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT);
    }

}
