<?php

/**
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_Helper_Api extends Mage_Core_Helper_Abstract
{
    /**
     * Checks if payment is available.
     *
     * @param int|string|null $storeId
     *
     * @return bool
     */
    public function isAvailable($storeId = null)
    {
        $token = Mage::getStoreConfig('glopal_checkout/general/glopal_api_key', $storeId);
        $shortName = Mage::getStoreConfig('webinterpret_connector/short_name', $storeId);

        return !empty($shortName) && !empty($token);
    }

    /**
     * Make POST Http call to Glopal API's.
     *
     * @param string $url
     * @param string $token
     * @param array $body
     *
     * @return false|array
     * @throws Exception
     */
    public function makePostHttpCall($url, $token = null, $body = null)
    {
        $rawData = Mage::helper('core')->jsonEncode($body);
        $response = $this->makeCall(
            array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_MAXREDIRS => 0,
                CURLOPT_TIMEOUT => 15,
                CURLOPT_SSL_VERIFYHOST => false,
                CURLOPT_FAILONERROR => true,
                CURLOPT_VERBOSE => true,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => $rawData,
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: application/json;charset=UTF-8',
                    'Content-Length: ' . strlen($rawData),
                    'Authorization: Bearer ' . $token
                ),
            )
        );

        return Mage::helper('core')->jsonDecode($response);
    }

    /**
     * Make GET Http call to Glopal API's.
     *
     * @param string $url
     * @param string $token
     *
     * @return false|array
     * @throws Exception
     */
    public function makeGetHttpCall($url, $token = null)
    {
        $response = $this->makeCall(
            array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_MAXREDIRS => 0,
                CURLOPT_TIMEOUT => 15,
                CURLOPT_SSL_VERIFYHOST => false,
                CURLOPT_FAILONERROR => true,
                CURLOPT_VERBOSE => true,
                CURLOPT_CUSTOMREQUEST => 'GET',
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: application/json;charset=UTF-8',
                    'Authorization: Bearer ' . $token
                ),
            )
        );

        return Mage::helper('core')->jsonDecode($response);
    }

    /**
     * Make CURL request based on given data.
     *
     * @param array $curlData
     *
     * @return bool|string
     * @throws Exception
     */
    private function makeCall($curlData)
    {
        $curl = curl_init();
        $verbose = fopen('php://temp', 'w+');
        $curlData[CURLOPT_STDERR] = $verbose;
        curl_setopt_array($curl, $curlData);

        $response = curl_exec($curl);

        $err = curl_error($curl);
        if ($err) {
            $errno = curl_errno($curl);
            $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            rewind($verbose);
            $log = stream_get_contents($verbose);

            curl_close($curl);
            fclose($verbose);

            throw new \Exception("Request error ({$errno}): {$err}\n\n{$log}", $status);
        }

        curl_close($curl);
        fclose($verbose);

        return $response;
    }
}
