<?php

/**
 * Default helper
 *
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_Helper_Data extends Mage_Core_Helper_Abstract
{
    public function installBridgeHelper()
    {
        $repo = Mage::getStoreConfig('webinterpret_connector/file_repository');
        $dir = $this->getModuleBridgeDir();

        // Install helper.php
        $remoteFile = $repo . DS . 'bridge2cart' . DS . $this->getExtensionVersion() . DS . 'helper.php';
        $contents = $this->downloadFile($remoteFile);
        if ($contents === false) {
            return false;
        }
        $localFile = $dir . DS . 'helper.php';
        $contents = str_replace('exit();', '', $contents);
        if (@file_put_contents($localFile, $contents) === false) {
            return false;
        }

        return true;
    }

    public function downloadFile($url, $timeout = 10)
    {
        try {
            $method = $this->getDownloadMethod();

            if ($method == 'stream') {
                $ctx = stream_context_create(array(
                        'http' =>
                            array(
                                'timeout' => $timeout,
                            ),
                        'https' =>
                            array(
                                'timeout' => $timeout,
                            ),
                    )
                );
                $contents = @file_get_contents($url, false, $ctx);
                if ($contents !== false) {
                    return $contents;
                }
            }

            if ($method == 'curl') {

                $ch = curl_init();
                $headers = array(
                    "Accept: */*",
                );
                curl_setopt_array($ch, array(
                    CURLOPT_RETURNTRANSFER => 1,
                    CURLOPT_URL => $url,
                    CURLOPT_TIMEOUT => $timeout,
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_ENCODING => '',
                    CURLOPT_HTTPHEADER => $headers,
                ));

                $contents = curl_exec($ch);

                if (curl_errno($ch)) {
                    curl_close($ch);
                    return false;
                }

                curl_close($ch);
                return $contents;
            }
        } catch (Exception $e) {
            return false;
        }

        return false;
    }

    public function validateGlopalKey($key)
    {
        return !empty($key) && strlen($key) == 32;
    }

    public function isEnabled()
    {
        return Mage::getStoreConfigFlag('webinterpret_connector/enabled');
    }

    public function isJsonLdMetadataEnabled()
    {
        return Mage::getStoreConfigFlag('webinterpret_connector/jsonld_metadata_enabled');
    }

    public function getGlopalKey()
    {
        $key = Mage::getStoreConfig('webinterpret_connector/key');
        $key2 = trim($key);
        if ($key != $key2) {
            Mage::getConfig()->saveConfig('webinterpret_connector/key', $key);
        }
        if ( empty( $key2 ) ) {
            $key2 = md5( uniqid( mt_rand(), true ) );
            Mage::getConfig()->saveConfig( 'webinterpret_connector/key', $key2 );
            Mage::getConfig()->reinit();
        }
        return (string)$key2;
    }

    public function isInstallationMode()
    {
        return Mage::getStoreConfigFlag('webinterpret_connector/installation_mode');
    }

    public function isAutomaticUpdatesEnabled()
    {
        return Mage::getStoreConfigFlag('webinterpret_connector/automatic_updates_enabled');
    }

    public function isGlobalNotificationsEnabled()
    {
        return Mage::getStoreConfigFlag('webinterpret_connector/global_notifications_enabled');
    }

    public function getRemoteAssetsUrl()
    {
        return Mage::getStoreConfig('webinterpret_connector/remote_assets_url');
    }

    public function isSignatureVerificationEnabled()
    {
        return Mage::getStoreConfigFlag('webinterpret_connector/signature_verification');
    }

    public function getModuleBridgeDir()
    {
        $dir = Mage::getModuleDir('controllers', 'Glopal_Connector');
        $dir = realpath($dir . '/../bridge2cart');
        return $dir;
    }

    public function getStoreBaseUrl()
    {
        if (Mage::app()->getStore()->isCurrentlySecure()) {
            return Mage::getStoreConfig('web/secure/base_url');
        }
        return Mage::getStoreConfig('web/unsecure/base_url');
    }

    /**
     * Check if cookie restriction notice should be displayed
     *
     * Always return false for pre-1.7 magento
     *
     * @return bool
     */
    public function isUserNotAllowSaveCookie()
    {
        if (Mage::getVersion() < '1.7.0.0') {
            return false;
        }

        return Mage::helper('core/cookie')->isUserNotAllowSaveCookie();
    }

    public function getMagentoVersionString()
    {
        $modules = (array)Mage::getConfig()->getNode('modules')->children();
        $edition = (array_key_exists('Enterprise_Enterprise', $modules)) ? 'Enterprise Edition' : 'Community Edition';
        $version = Mage::getVersion();
        return "Magento {$edition} {$version}";
    }

    public function getMagentoVersionNumber()
    {
        return Mage::getVersion();
    }

    public function getDownloadMethod()
    {
        if (ini_get('allow_url_fopen')) {
            return 'stream';
        }
        if (function_exists('curl_version')) {
            return 'curl';
        }

        return false;
    }

    public function isOpenSslEnabled()
    {
        return function_exists('openssl_verify');
    }

    public function isCurlEnabled()
    {
        return function_exists('curl_version');
    }

    public function getPackageContent()
    {
        $path = Mage::getModuleDir('etc', 'Glopal_Connector') . DS . 'content.json';
        if (is_readable($path)) {
            return json_decode(file_get_contents($path), true);
        }
        return false;
    }

    public function validatePackageFiles($files)
    {
        $errors = [];
        foreach ($files as $file) {
            $path = Mage::getBaseDir() . DS . $file['path'];

            if ($file['type'] == 'file') {
                if (!file_exists($path)) {
                    $errors['missing'][] = $path;
                } elseif (!is_readable($path)) {
                    $errors['unreadable'][] = $path;
                }
                if ($file['hash'] && $file['hash'] !== md5_file($path)) {
                    $errors['corrupted'][] = $path;
                }
                if ($this->isAutomaticUpdatesEnabled()) {
                    if (!is_writeable($path)) {
                        // Attempt repair
                        @chmod($path, 0664);
                    }
                }
                if (!is_writeable($path)) {
                    $errors['unwritable'][] = $path;
                }
            }

            if ($file['type'] == 'dir') {
                if (file_exists($path) && $this->isAutomaticUpdatesEnabled()) {
                    if (!is_writeable($path)) {
                        // Attempt repair
                        @chmod($path, 0775);
                    }
                }
                if (!is_writeable($path)) {
                    $errors['unwritable'][] = $path;
                }
            }
        }
        return $errors;
    }

    public function selfTest()
    {
        $report = array();

        // Glopal Key
        $report['webinterpret_key']['label'] = $this->__('Glopal Key');
        $key = $this->getGlopalKey();
        if ($this->validateGlopalKey($key)) {
            $report['webinterpret_key']['info'] = $this->__('Key is valid');
        } else {
            $report['webinterpret_key']['errors'] = array();
            if (empty($key)) {
                $report['webinterpret_key']['errors'][] = $this->__('Key is missing.');
            } else {
                $report['webinterpret_key']['errors'][] = $this->__('Incorrect key.');
            }
        }

        // Integration
        $report['integration']['label'] = $this->__('Integration');
        if ($this->isEnabled()) {
            $report['integration']['info'] = $this->__('Extension is ON');
        } else {
            $report['integration']['errors'] = array();
            $report['integration']['errors'][] = $this->__('Integration is turned OFF.');
        }

        // Plugin version
        $report['plugin_version']['label'] = $this->__('Extension version');
        $report['plugin_version']['info'] = 'Glopal Connector ' . $this->getExtensionVersion();

        // PHP Version
        $report['php_version']['label'] = $this->__('PHP');
        $report['php_version']['info'] = phpversion();
        if (version_compare(phpversion(), '5.4', '<')) {
            $report['php_version']['errors'] = array();
            $report['php_version']['errors'][] = $this->__('Unsupported PHP version. Please upgrade to version 5.4 or higher.');
        }

        // Magento version
        $report['magento_version']['label'] = $this->__('Magento');
        $report['magento_version']['info'] = $this->getMagentoVersionString();
        if (version_compare($this->getMagentoVersionNumber(), '1.6', '<')) {
            $report['magento_version']['errors'] = array();
            $report['magento_version']['errors'][] = $this->__('Unsupported Magento version. Please upgrade to version 1.6 or higher.');
        }

        // OpenSSL
        $report['openssl']['label'] = $this->__('OpenSSL');
        if ($this->isOpenSslEnabled()) {
            $report['openssl']['info'] = OPENSSL_VERSION_TEXT;
        } else {
            $report['openssl']['errors'] = array();
            $report['openssl']['errors'][] = $this->__('OpenSSL extension not found. Please install OpenSSL on this server.');
        }

        // cURL
        $report['curl']['label'] = $this->__('cURL');
        if ($this->isCurlEnabled()) {
            $curlVersion = curl_version();

            $curlInfo = $curlVersion['version'];
            if (defined('CURL_VERSION_HTTP2') && $curlVersion['features'] & CURL_VERSION_HTTP2) {
                $curlInfo .= ' (HTTP/2 enabled)';
            }
            $report['curl']['info'] = $curlInfo;

            if (!($curlVersion['features'] & CURL_VERSION_SSL)) {
                $report['curl']['errors'][] = $this->__('cURL does not have SSL support');
            }

            if (!in_array('https', $curlVersion['protocols'])) {
                $report['curl']['errors'][] = $this->__('cURL does not have HTTPS support');
            }
        } else {
            $report['curl']['errors'][] = $this->__('cURL extension not found. Please install cURL on this server.');
        }

        // Remote files
        $report['download_method']['label'] = $this->__('Download Method');
        $method = $this->getDownloadMethod();
        if ($method == false) {
            $report['download_method']['errors'] = array();
            $report['download_method']['errors'][] = $this->__('No supported download method was found. Please enable allow_url_fopen in php.ini or install cURL on this server.');
        } else {
            $report['download_method']['info'] = $this->__('Using %s', $method);
        }

        // Extension files
        $report['extension_files']['label'] = $this->__('Extension Files');
        if ($this->isAutomaticUpdatesEnabled()) {
            $report['auto_update']['label'] = $this->__('Automatic Updates');
        }

        // Check file integrity & permissions
        $files = $this->getPackageContent();
        if (!$files) {
            $report['extension_files']['errors'][] =
                $this->__("Package content file is missing or not readable.<br>Please reinstall this extension using Magento Connect Manager.");
        }
        $errors = $this->validatePackageFiles($files);
        if (isset($errors['missing'])) {
            $report['extension_files']['errors'][] = $this->__("Files are missing:<br><small>%s</small><br>Please upload files or reinstall this extension using Magento Connect Manager.",
                join('<br>', $errors['missing']));
        }
        if (isset($errors['unreadable'])) {
            $report['extension_files']['errors'][] = $this->__("Files are not readable:<br><small>%s</small><br>Please update file permissions.",
                join('<br>', $errors['unreadable']));
        }
        if (isset($errors['corrupted'])) {
            $report['extension_files']['errors'][] = $this->__("Files might be corrupted:<br><small>%s</small><br>Please upload the file or reinstall this extension using Magento Connect Manager.",
                join('<br>', $errors['corrupted']));
        }
        if (isset($errors['unwritable'])) {
            $report['auto_update']['errors'][] = $this->__("Files or directories are not writable:<br><small>%s</small><br>Please update file permissions.",
                join('<br>', $errors['unwritable']));
        }
        if (!isset($report['auto_update']['errors'])) {
            $report['auto_update']['info'] = $this->__('Enabled');
        }
        if (!isset($report['extension_files']['errors'])) {
            $report['extension_files']['info'] = $this->__('Extension is installed');
        }

        return $report;
    }

    public function getExtensionVersion()
    {
        return (string)Mage::getConfig()->getNode()->modules->Glopal_Connector->version;
    }

    public function getCheckoutUrl()
    {
        $url = Mage::getStoreConfig('webinterpret_connector/checkout_url');
        if (empty($url)) {
            $url = Mage::helper('checkout/url')->getCheckoutUrl();
        }

        $context = new Varien_Object();
        $context->setUrl($url);

        Mage::dispatchEvent(
            'webinterpret_connector_get_checkout_url', array('context' => $context)
        );

        $result = $context->getUrl();
        if (!empty($result)) {
            $url = $result;
        }

        return $url;
    }

    public function getSecretKey($generate = true)
    {
        $key = Mage::getStoreConfig('webinterpret_connector/secret_key');
        if (empty($key) && $generate) {
            $key = $this->generateSecretKey();
            $this->setSecretKey($key);
        }
        return $key;
    }

    public function setSecretKey($key)
    {
        Mage::getConfig()->saveConfig('webinterpret_connector/secret_key', $key);
        Mage::getConfig()->reinit();
    }

    public function generateSecretKey()
    {
        return base64_encode(openssl_random_pseudo_bytes(openssl_cipher_iv_length('AES-256-CBC')));
    }

    /**
     * Is passing magento DB link to bridge file enabled
     */
    public function isPassDbLinkEnabled()
    {
        return Mage::getStoreConfigFlag('webinterpret_connector/pass_db_link');
    }

    /**
     * @param $url
     * @param $data  Encoded POST data
     * @return mixed  Response body
     *
     * @throws Exception
     */
    public function sendPostRequest($url, $data)
    {
        $curl = curl_init();
        $verbose = fopen('php://temp', 'w+');

        curl_setopt_array($curl, array(
          CURLOPT_URL => $url,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_MAXREDIRS => 0,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_SSL_VERIFYHOST => false,
          CURLOPT_FAILONERROR => true,
          CURLOPT_VERBOSE => true,
          CURLOPT_STDERR => $verbose,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_POSTFIELDS => $data,
          CURLOPT_HTTPHEADER => array(
            'Content-Type: application/json;charset=UTF-8',
            'Content-Length: ' . strlen($data)
          ),
        ));

        $response = curl_exec($curl);

        $err = curl_error($curl);
        if ($err) {
            $errno = curl_errno($curl);
            $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            rewind($verbose);
            $log = stream_get_contents($verbose);

            curl_close($curl);
            fclose($verbose);

            throw new \Exception("Request error ({$errno}): {$err}\n\n{$log}", $status);
        }

        curl_close($curl);
        fclose($verbose);

        return $response;
    }

    private function quoteFilterRecursive(&$data) {
        foreach ($data as $key => &$value) {
            if ($key === 'uenc' || $key === 'info_buyRequest' || substr($key, 0, 6) === '_cache') {
                unset($data[$key]);
            } elseif (is_array($value)) {
                $data[$key] = $this->quoteFilterRecursive($value);
            }
        }

        return $data;
    }
    
    private function formatOrderOptions(&$orderOptions, &$simpleProduct)
    {
        $data = array();
        if ($orderOptions) {
            $data = $orderOptions;
        }
        if ($simpleProduct) {
            $data['wi_variant_id'] = $simpleProduct->getId();
            $data['wi_variant_weight'] = $simpleProduct->getWeight();
        }

        if (!empty($orderOptions['attributes_info'])) {
            $attributes = $orderOptions['attributes_info'];

            if (!empty($orderOptions['info_buyRequest']['super_attribute'])) {
                $i = 0;
                foreach ($orderOptions['info_buyRequest']['super_attribute'] as $key => $value) {
                    $attributes[$i]['option_id'] = $key;
                    $attributes[$i]['value_id'] = $value;
                    $i++;
                }
            }

            $data['wi_attributes_merged'] = $attributes;
        }

        return empty($data) ? null : $data;
    }

    public function buildVersion2Data($quote, $encode=true)
    {
        $hsCodeAttributeCode = Mage::getStoreConfig('glopal_cart_transfer/product_data/magento_attibute_hs_code');
        if(empty($hsCodeAttributeCode)) {
            $hsCodeAttributeCode = 'hs_code';
        }

        $cooCodeAttributeCode = Mage::getStoreConfig('glopal_cart_transfer/product_data/magento_attibute_coo');
        if(empty($cooCodeAttributeCode)) {
            $cooCodeAttributeCode = 'country_of_manufacture';
        }

        $quoteItems = array();

        foreach ($quote->getAllVisibleItems() as $item) {
            $modelProduct = $item->getProduct();
            $product = $modelProduct->getTypeInstance(true);
            $quoteItemData = $item->getData();

            // Variant is simple product that belongs to configurable
            $simpleProduct = null;
            $simpleOption = $item->getOptionByCode('simple_product');
            if ($simpleOption) {
               $simpleProduct = $simpleOption->getProduct();
            }

            $quoteItemData['wi_product'] = $modelProduct->getData();

            $productId = null !== $simpleProduct ? $simpleProduct->getId() : $item->getProductId();
            $quoteItemData['wi_product']['hs_code'] = $this->getProductAttribute(
                $productId,
                $item->getStoreId(),
                $hsCodeAttributeCode
            );
            $quoteItemData['wi_product']['country_id_of_origin'] = $this->getProductAttribute(
                $productId,
                $item->getStoreId(),
                $cooCodeAttributeCode
            );

            $quoteItemData['wi_order_options'] = $this->formatOrderOptions(
                $product->getOrderOptions($modelProduct),
                $simpleProduct
            );
            $quoteItemData['wi_buy_request'] = $item->getBuyRequest()->getData();
            $quoteItems[] = $quoteItemData;
        }
        
        $taxOptions = [
            'price_includes_tax' => Mage::getStoreConfig('tax/calculation/price_includes_tax', $quote->getStoreId()),
            'algorithm' => Mage::getStoreConfig('tax/calculation/algorithm', $quote->getStoreId()),
            'apply_after_discount' => Mage::getStoreConfig('tax/calculation/apply_after_discount', $quote->getStoreId()),
            'discount_tax' => Mage::getStoreConfig('tax/calculation/discount_tax', $quote->getStoreId()),
            'apply_tax_on' => Mage::getStoreConfig('tax/calculation/apply_tax_on', $quote->getStoreId()),
            'cross_border_trade_enabled' => Mage::getStoreConfig('tax/calculation/cross_border_trade_enabled', $quote->getStoreId()),
            'tax_display_shipping' => Mage::getStoreConfig('tax/display/shipping', $quote->getStoreId()),
            'tax_display_type' => Mage::getStoreConfig('tax/display/type', $quote->getStoreId()),
            'tax_calculation_shipping_includes_tax' => Mage::getStoreConfig('tax/calculation/shipping_includes_tax', $quote->getStoreId()),
        ];

        $quoteData = $quote->getData();
        $quoteData['wi_items'] = $quoteItems;
        $quoteData['wi_tax_options'] = $taxOptions;

        // Force serialization to array and filter unwanted
        $quoteData = json_decode(json_encode($quoteData), true);
        $quoteData = $this->quoteFilterRecursive($quoteData);

        $quoteData['wi_checkout'] = $this->getCheckoutData($quote->getStoreId());

        $moduleVersion = (string) Mage::getConfig()->getNode()->modules->Glopal_Connector->version;
        $response = array(
            'magento_version' => Mage::getVersion(),
            'connector_version' => $moduleVersion,
        );

        if ($encode) {
            $encodedQuoteData = base64_encode(json_encode($quoteData));
            $key = Mage::helper('glopal_connector')->getGlopalKey();
            $response['quote'] = $encodedQuoteData;
            $response['hash'] = hash_hmac('sha256', $encodedQuoteData, $key);
        } else {
            $response['quote'] = $quoteData;
        }

        if (isset($_GET['debug'])) {
            $response['quote_debug'] = $quoteData;
        }

        return $response;
    }

    /**
     * @param int          $productId
     * @param int          $storeId
     * @param array|string $attributes
     *
     * @return mixed
     */
    public function getProductAttribute($productId, $storeId, $attributes)
    {
        if (!is_array($attributes)) {
            $attributes = array($attributes);
        }
        
        $select_inputs = array('select', 'multiselect');
        $attribute_collection = Mage::getModel('eav/config')
            ->getEntityType(Mage_Catalog_Model_Product::ENTITY)
            ->getAttributeCollection()
            ->addFieldToFilter('attribute_code', array('in' => $attributes))
        ;
        $attribute_data = array();
        foreach($attribute_collection as $attribute) {
            $attribute_data[$attribute->getAttributeCode()] = $attribute;
        }
        
        $productCollection = Mage::getResourceModel('catalog/product_collection');
        foreach ($attributes as $attribute) {
            $productCollection->addAttributeToSelect($attribute);
        }
        $productCollection->addAttributeToFilter('entity_id', $productId);
        $productCollection->addStoreFilter($storeId);

        $product = $productCollection->load()->getFirstItem();
        $result  = null;
        foreach ($attributes as $attribute) {
            /* @var $product Mage_Catalog_Model_Product */
            if($attribute == 'country_of_manufacture') {
                $cof = $product->getData($attribute);
                if(empty($cof)) {
                    $cof = trim($product->getAttributeText($attribute));
                }
                if(empty($cof)) {
                    $product->load($product->getId());
                    $cof = $product->getData($attribute);
                    if(empty($cof)) {
                        $cof = trim($product->getAttributeText($attribute));
                    }
                }
                $result = $cof;
            } elseif(isset($attribute_data[$attribute]) && is_numeric($attribute_data[$attribute]) && (in_array($attribute_data[$attribute]->getFrontendInput(), $select_inputs))) {
                $result = $product->getAttributeText($attribute);
            } else {
                $result = $product->getDataUsingMethod($attribute);
            }
            if ($result) {
                break;
            }
        }

        return $result;
    }

    public function getCartTransferGetParams()
    {
        $result = array(
            'referrer' => Mage::getUrl('checkout/cart'),
            'visitorCurrency' => Mage::app()->getStore()->getCurrentCurrencyCode(),
            'visitorLanguage' => str_replace('_', '-', Mage::app()->getLocale()->getLocaleCode())
        );

        $params = $this->getCartTransferAnalyticsParams();
        foreach ($params as $paramType => $param) {
            if ($paramType === 'cookies') {
                foreach ($param as $cookieName => $cookieValue) {
                    $result['tracking[cookies][' . $cookieName . ']'] = $this->urlEncodeValue($cookieValue);
                }
            }

            if ($paramType === 'ga') {
                $result['clientIds[ga]'] = $this->urlEncodeValue($param);
            }

            if ($paramType === 'ga4') {
                $result['clientIds[ga4_session_id]'] = $this->urlEncodeValue($param);
            }
        }

        return $result;
    }

    public function getCartTransferAnalyticsParams()
    {
        // Tracking cookies
        $result = array();
        $cookieManager = Mage::getSingleton('core/cookie');

        foreach (['_gcl_aw', '_gcl_dc', ' _gcl_gb', '_gcl_au', '_fbc', '_fbp'] as $cookie) {
            $value = $cookieManager->get($cookie);

            if (null !== $value && false !== $value) {
                $result['cookies'][$cookie] = $value;
            }
        }

        // GA cookies
        $gaId = $cookieManager->get('_ga');
        if (null !== $gaId) {
            $gaParts = explode('.', $gaId);
            if (isset($gaParts[2], $gaParts[3])) {
                $result['ga'] = $gaParts[2] . '.' . $gaParts[3];
            }
        }

        // GA4 cookies
        $ga4MeasurementId = Mage::getStoreConfig('glopal_checkout/analytics/ga4_measurement_id');
        if (!empty($ga4MeasurementId)) {
            $ga4Id =  $cookieManager->get('_ga_' . $ga4MeasurementId);

            if (null !== $ga4Id) {
                $ga4Parts = explode('.', $ga4Id);
                if (isset($ga4Parts[2])) {
                    $result['ga4'] = $ga4Parts[2];
                }
            }
        }

        return $result;
    }

    /**
     * Encoding url value.
     *
     * @param string $value
     *
     * @return string
     */
    private function urlEncodeValue($value)
    {
        return urlencode($value);
    }

    /**
     * Provide checkout data.
     *
     * @param null|string|int $storeId
     *
     * @return array
     */
    private function getCheckoutData($storeId = null)
    {
        $checkoutPurchaseUrl = Mage::getStoreConfig('glopal_cart_transfer/callback_urls/checkout_purchase', $storeId);
        if (empty($checkoutPurchaseUrl)) {
            $cartId = Mage::getSingleton('checkout/cart')->getQuote()->getId();
            if (empty($cartId)) {
                return [];
            }

            $secretKey = Mage::getStoreConfig('webinterpret_connector/key');
            /** @var Glopal_Connector_Helper_Utils $utilsHelper */
            $utilsHelper = Mage::helper('glopal_connector/utils');
            $checkoutPurchaseUrl = Mage::getUrl('glopal_connector/callback/purchase')
                . '?hmac='
                . $utilsHelper->buildHmacKey($cartId, $secretKey);
        }

        return [
            'inventory_check' => '',
            'checkout_purchase' => $checkoutPurchaseUrl
        ];
    }
}
