<?php

/**
 * DB Link helper
 *
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */

class Glopal_Connector_Helper_Dblink
{
    public $noResult = array('delete', 'update', 'move', 'truncate', 'insert', 'set', 'create', 'drop', 'replace');
    public $dataBaseHandle = null;

    private $_insertedId = 0;
    private $_affectedRows = 0;

    /**
     * @return bool|null|PDO
     */
    private function getDataBaseHandle()
    {
        if ($this->dataBaseHandle) {
            return $this->dataBaseHandle;
        }

        $this->dataBaseHandle = Mage::getSingleton('core/resource')->getConnection('core_write');

        if (!$this->dataBaseHandle) {
            exit('[ERROR] MySQL Query Error: Can not connect to DB');
        }

        return $this->dataBaseHandle;
    }

    /**
     * @param string $sql sql query
     *
     * @return array|bool
     */
    public function localQuery($sql)
    {
        $result = array();
        $dataBaseHandle = $this->getDataBaseHandle();

        $sth = $dataBaseHandle->query($sql);

        foreach ($this->noResult as $statement) {
            if (!$sth || strpos(strtolower(trim($sql)), $statement) === 0) {
                return true;
            }
        }

        $result = $sth->fetchAll();

        return $result;
    }

    /**
     * @param string $sql       sql query
     * @param int    $fetchType fetch Type
     *
     * @return array
     */
    public function query($sql, $fetchType)
    {
        $result = array(
            'result'        => null,
            'message'       => '',
            'fetchedFields' => array()
        );
        $dataBaseHandle = $this->getDataBaseHandle();

        if (isset($_GET['disable_checks'])) {
            $this->localQuery('SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0');
            $this->localQuery("SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO'");
        }

        if (isset($_REQUEST['set_names'])) {
            $this->localQuery("SET NAMES '" . ($_REQUEST['set_names']) . "'");
            $this->localQuery("SET CHARACTER SET '" . ($_REQUEST['set_names']) . "'");
            $this->localQuery("SET CHARACTER_SET_CONNECTION = '" . ($_REQUEST['set_names']) . "'");
        }

        try {
            $res = $dataBaseHandle->query($sql);
            switch ($fetchType) {
                case 3:
                    $res->setFetchMode(Zend_Db::FETCH_BOTH);
                    break;
                case 2:
                    $res->setFetchMode(Zend_Db::FETCH_NUM);
                    break;
                case 1:
                default:
                    $res->setFetchMode(Zend_Db::FETCH_ASSOC);
                    break;
            }
            $this->_affectedRows = $res->rowCount();
            $this->_insertedId = $dataBaseHandle->lastInsertId();
        } catch (PDOException $e) {
            $result['message'] = '[ERROR] Mysql Query Error: ' . $e->getCode() . ', ' . $e->getMessage();
            return $result;
        }

        foreach ($this->noResult as $statement) {
            if (!$res || strpos(strtolower(trim($sql)), $statement) === 0) {
                $result['result'] = true;
                return $result;
            }
        }

        $rows = $res->fetchAll();

        if (isset($_GET['disable_checks'])) {
            $this->localQuery("SET SQL_MODE=IFNULL(@OLD_SQL_MODE,'')");
            $this->localQuery("SET FOREIGN_KEY_CHECKS=IFNULL(@OLD_FOREIGN_KEY_CHECKS,0)");
        }

        $result['result'] = $rows;

        unset($res);
        return $result;
    }

    /**
     * @return string|int
     */
    public function getLastInsertId()
    {
        return $this->_insertedId;
    }

    /**
     * @return int
     */
    public function getAffectedRows()
    {
        return $this->_affectedRows;
    }

    /**
     * @return  void
     */
    public function __destruct()
    {
        $this->dataBaseHandle = null;
    }
}
