<?php

use Glopal_Connector_Model_SignatureException as SignatureException;

/**
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_Helper_Verifier extends Mage_Core_Helper_Abstract
{
    /**
     * Verifies if the current request has been correctly signed by Glopal
     *
     * @return bool
     * @throws Exception
     */
    public static function verifyRequestWebInterpretSignature()
    {
        if (!Mage::helper('glopal_connector')->isSignatureVerificationEnabled()) {
            return true;
        }

        if (!function_exists('openssl_verify')) {
            throw new SignatureException('OpenSSL is required.');
        }

        if (empty($_SERVER['HTTP_WEBINTERPRET_REQUEST_ID']) || empty ($_SERVER['HTTP_WEBINTERPRET_SIGNATURE']) || empty ($_SERVER['HTTP_WEBINTERPRET_SIGNATURE_VERSION'])) {
            throw new SignatureException('Request needs to be signed by Glopal.');
        }

        self::filter_allowed_keys($_GET);
        self::filter_allowed_keys($_POST);

        $requestId = $_SERVER['HTTP_WEBINTERPRET_REQUEST_ID'];
        $signature = $_SERVER['HTTP_WEBINTERPRET_SIGNATURE'];
        $signature = base64_decode($signature);

        $version = $_SERVER['HTTP_WEBINTERPRET_SIGNATURE_VERSION'];

        if ($version == 2) {
            // Query data
            $queryArray = $_GET;
            ksort($queryArray);
            $queryData = serialize($queryArray);

            // Post data
            $postData = $_POST;
            ksort($postData);
            $postData = serialize($postData);

            $data = $requestId.$queryData.$postData;
        } else {
            throw new SignatureException('Not supported signature version.');
        }

        // Verify signature
        $publicKey = Mage::getStoreConfig('webinterpret_connector/public_key');

        $ok = openssl_verify($data, $signature, $publicKey, OPENSSL_ALGO_SHA1);

        if ($ok == 1) {
            return true;
        }
        if ($ok == 0) {
            throw new SignatureException('Incorrect signature.');
        }

        throw new SignatureException('Error checking signature');
    }

    /**
     * Leave only original keys so that signature verification can pass
     */
    private static function filter_allowed_keys( &$array ) {
        if ( isset( $array['allowed_keys'] ) ) {
            $allowedKeys   = explode( ',', $array['allowed_keys'] );
            $allowedKeys[] = 'allowed_keys';
            $array         = array_intersect_key( $array, array_flip( $allowedKeys ) );
        }
    }
}