<?php
/**
 * Glopal payment
 *
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_Model_Payment_Glopal extends Mage_Payment_Model_Method_Abstract
{
    const PAYMENT_METHOD_GLOPAL_CODE = 'glopal';
    const ORDER_GLOPAL_STATUS = 'pending_glopal';

    /**
     * Payment method code
     *
     * @var string
     */
    protected $_code = self::PAYMENT_METHOD_GLOPAL_CODE;

    /**
     * Hide payment method on checkout page.
     *
     * @var bool
     */
    protected $_canUseCheckout = true;

    /**
     * Returns tru for creating an order.
     *
     * @var bool
     */
    protected $_canOrder = true;

    /**
     * Initialized flag status.
     * Always true for Glopal.
     *
     * @var bool
     */
    protected $_isInitializeNeeded = true;

    /**
     * Payment info block
     *
     * @var string
     */
    protected $_infoBlockType = 'glopal_connector/payment_info';

    /**
     * Rewrite original method isAvailable().
     * Returns always true.
     *
     * @param Mage_Sales_Model_Quote|null $quote
     *
     * @return bool
     */
    public function isAvailable($quote = null)
    {
        return true;
    }

    /**
     * Can be used in regular checkout
     *
     * @return bool
     * @throws Mage_Core_Model_Store_Exception
     */
    public function canUseCheckout()
    {
        $storeId = Mage::app()->getStore()->getStoreId();
        /** @var Glopal_Connector_Helper_Api $helper */
        $helper = Mage::helper('glopal_connector/api');
        $useGlopalCheckout = Mage::getStoreConfigFlag('glopal_checkout/general/use_glopal_checkout', $storeId);
        $useGlopalPayment = Mage::getStoreConfigFlag('glopal_checkout/general/use_glopal_payment', $storeId);

        return !$useGlopalCheckout && $useGlopalPayment && $helper->isAvailable($storeId);
    }

    /**
     * Method that will be executed instead of authorize or capture
     * if flag isInitializeNeeded set to true
     *
     * @param string $paymentAction
     * @param object $stateObject
     *
     * @return $this
     */
    public function initialize($paymentAction, $stateObject)
    {
        $stateObject->setState(Mage_Sales_Model_Order::STATE_NEW);
        $stateObject->setStatus(self::ORDER_GLOPAL_STATUS);
        $stateObject->setIsNotified(false);

        return $this;
    }

    /**
     * Return Order place redirect url
     *
     * @return string
     * @throws Exception
     */
    public function getOrderPlaceRedirectUrl()
    {
        return Mage::getUrl('glopal_connector/payment/redirect', array('_secure' => true));
    }
}
