<?php

class Glopal_Connector_AbstractController extends Mage_Core_Controller_Front_Action
{
    public function preDispatch()
    {
        $actionName = $this->getRequest()->getActionName();
        $actionName = strtolower($actionName);
        parent::preDispatch();
        try {
            switch ($actionName) {
                case 'v1':
                default:
                    Glopal_Connector_Helper_Verifier::verifyRequestWebInterpretSignature();
                    break;
            }
        } catch (Glopal_Connector_Model_SignatureException $e) {
            $this->send([], true, $e->getMessage(), 403);
        } catch (Exception $e) {
            $this->send([], true, $e->getMessage(), 403);
        }
        return $this;
    }

    /**
     * @param array $data
     * @param bool $error
     * @param string $message
     * @param bool|int $responseCode
     */
    protected function send($data, $error = false, $message = '', $responseCode = false)
    {
        $result = [
            'error' => $error,
            'message' => $message,
            'data' => $data
        ];
        $this->getResponse()->setBody(
            Mage::helper('core')->jsonEncode($result)
        );
        $this->getResponse()->clearHeaders()->setHeader('Content-type', 'application/json', true);
        if (is_numeric($responseCode)) {
            $this->getResponse()->setHttpResponseCode($responseCode);
        }
        $this->getResponse()->sendResponse();
        exit;
    }

    public function v1Action()
    {
        try {
            $this->send($this->process());
        } catch (BadFunctionCallException $e) {
            $this->send([], true, $e->getMessage(), 500);
        } catch (LogicException $e) {
            $this->send([], true, $e->getMessage(), 400);
        } catch (Exception $e) {
            $this->send([], true, $e->getMessage(), 500);
        }
    }

    /**
     * @return array
     */
    protected function process()
    {
        return [];
    }
}
