<?php
/**
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */

class Glopal_Connector_Adminhtml_GlopalController extends Mage_Adminhtml_Controller_Action
{
    private $_instalationHelper = null;

    /**
     * Index action
     */
    public function indexAction()
    {
        $this->_forbidden();
    }

    /**
     * View or save configuration
     */
    public function configAction()
    {
        if( $this->getRequest()->isPost() ) {
            $this->_saveConfig();
        } else {
            $this->_viewConfig();
        }
    }

    /**
     * View configuration
     */
    private function _viewConfig()
    {
        $data = array();
        $fields = $this->getRequest()->getParam('fields');
        
        if( ! empty( $fields ) && is_array( $fields ) ) {
    
            $this->_validateFields( $fields );

            foreach( $fields as $field ) {

                $path = $field['path'];
                $scope = $field['scope'];
                $scopeId = $field['scope_id'];

                if( $scope == 'websites' ) {
                    $field['value'] = Mage::app()->getWebsite($scopeId)->getConfig( $path );
                } else {
                    $field['value'] = Mage::getStoreConfig( $path, $scopeId );
                }
                $data[]= $field;
            }
        }
        
        $this->getResponse()->clearHeaders()->setHeader( 'Content-type', 'application/json', true );
        $this->getResponse()->setBody( Mage::helper('core')->jsonEncode( $data ) );
    }

    /**
     * Save configuration
     */
    private function _saveConfig()
    {
        $body = $this->getRequest()->getRawBody();
        $fields = Mage::helper('core')->jsonDecode( $body );

        if( ! empty( $fields ) && is_array( $fields ) ) {

            $this->_validateFields( $fields, false );

            foreach( $fields as $field ) {

                $path = $field['path'];
                $scope = $field['scope'];
                $scopeId = $field['scope_id'];
                $value = $field['value'];

                if ($value === null) {
                    Mage::getConfig()->deleteConfig( $path, $scope, $scopeId );
                } else {
                    Mage::getConfig()->saveConfig($path, $value, $scope, $scopeId);
                }
            }

            Mage::getConfig()->reinit();
        }

        $this->getResponse()->clearHeaders()->setHeader( 'Content-type', 'application/json', true );
        $this->getResponse()->setBody( Mage::helper('core')->jsonEncode( ['status' => 'ok'] ) );
    }

    /**
     * Retrieve stores json
     */
    public function storesAction()
    {
        $data = $this->_instalationHelper()->getStoresData();
        
        $this->getResponse()->clearHeaders()->setHeader( 'Content-type', 'application/json', true );
        $this->getResponse()->setBody( Mage::helper('core')->jsonEncode( $data ) );
    }

    /**
     * Handle access level redirects
     */
    private function _forbidden()
    {
        $this->getResponse()->setHeader('HTTP/1.1','403 Forbidden');
        $this->loadLayout(array('default', 'adminhtml_denied'));
        $this->renderLayout();
    }

    /**
     * Get instalation helper instance
     */
    private function _instalationHelper()
    {
        if( $this->_instalationHelper === null ) {
            $this->_instalationHelper = Mage::helper('glopal_connector/instalation');
        }
        return $this->_instalationHelper;
    }

    /**
     * Check is allowed access to action
     * All that can access WI configuration can
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        return Mage::getSingleton('admin/session')->isAllowed('system/config/glopal_connector');
    }

    /**
     * Validate allowed field keys
     */
    private function _validateFields( $fields, $read = true )
    {
        $pathsRegEx = ($read)
            ? Mage::getStoreConfig('webinterpret_connector/adminview_readable_paths')
            : Mage::getStoreConfig('webinterpret_connector/adminview_writable_paths');

        foreach( $fields as $field ) {

            if( empty( $field['path'] ) || ! preg_match( '/' . $pathsRegEx . '/', $field['path'] ) ) {
                header('HTTP/1.1 401 Unauthorized');
                echo $this->__('Unauthorized to %s %s', $read ? 'read' : 'write', $field['path'] );
                exit();
            }
        }
    }
}
