<?php

/**
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_CallbackController extends Mage_Core_Controller_Front_Action
{
    /**
     * Return cart data in JSON
     */
    public function purchaseAction()
    {
        $response = $this->getResponse()->clearHeaders()->setHeader('Content-type', 'application/json', true);
        $responseData = array('status' => true, 'message' => '');

        try {
            $jsonBody = $this->getRequest()->getRawBody();
            $requestBody = Mage::helper('core')->jsonDecode($jsonBody);
            $requestCartId = isset($requestBody['cart']['external_id']) ? $requestBody['cart']['external_id'] : null;

            if (!empty($requestCartId) && $this->validateHmacKey($requestCartId)) {
                $quote = Mage::getModel('sales/quote')->load($requestCartId);
                foreach ($quote->getAllItems() as $item) {
                    $quote->removeItem($item->getId());
                }

                $quote->save();
                Mage::getSingleton('checkout/session')->clear();
                Mage::getSingleton('checkout/session')->setCartWasUpdated(true);
            } else {
                $errorMessage = 'Invalid or missing hmac key';
                $responseData['status'] = false;
                $responseData['message'] = $errorMessage;
                $response->setHttpResponseCode(400);
                Mage::log($errorMessage, Zend_Log::DEBUG);
            }
        } catch (Mage_Core_Exception $e) {
            $responseData['status'] = false;
            $responseData['message'] = $e->getMessage();
            $response->setHttpResponseCode(500);
            Mage::logException($e);
        } catch (Exception $e) {
            $responseData['status'] = false;
            $responseData['message'] = $e->getMessage();
            $response->setHttpResponseCode(500);
            Mage::logException($e);
        }

        $response->setBody(Mage::helper('core')->jsonEncode($responseData));

        return $response;
    }

    /**
     * Validate requested hmac key.
     *
     * @param string $cartId
     * @return bool
     */
    private function validateHmacKey($cartId)
    {
        $requestedHmacKey = $this->getRequest()->getParam('hmac');
        if (empty($requestedHmacKey)) {
            return false;
        }

        $secretKey = Mage::getStoreConfig('webinterpret_connector/key');
        /** @var Glopal_Connector_Helper_Utils $utilsHelper */
        $utilsHelper = Mage::helper('glopal_connector/utils');
        $expectedHmacKey = $utilsHelper->buildHmacKey($cartId, $secretKey);

        return hash_equals($expectedHmacKey, $requestedHmacKey);
    }
}
