<?php

/**
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_CartController extends Mage_Core_Controller_Front_Action
{
    /**
     * Return cart data in JSON
     */
    public function indexAction()
    {
        if ($this->getVersion() === 2) {
            $data = $this->buildVersion2Data();
        } else {
            $data = $this->buildVersion1Data();
        }
        header('Content-Type: application/json');
        echo json_encode($data);
        die();
    }

    /**
     * Get version number from URL parameter
     *
     * Version 1 = Initial
     * Version 2 = 2018.Q4
     */
    private function getVersion()
    {
        $version = 1;
        if (isset($_GET['v']) && is_numeric($_GET['v'])) {
            $version = intval($_GET['v']);
        }
        if ($version > 2) {
            $version = 2;
        }
        return $version;
    }

    /**
     * Return version 1 cart data
     *
     * Example requests:
     *
     * GET https://www.example.com/webinterpret_connector/cart
     * GET https://www.example.com/webinterpret_connector/cart?v=1
     *
     * Example response (JSON):
     * [
     *   {
     *     external_id: "403",
     *     variant_id: "501",
     *     quantity: 1,
     *     custom_options: null
     *   }
     * ]
     */
    private function buildVersion1Data()
    {
        $cart = array();
        $quote = Mage::getSingleton('checkout/cart')->getQuote();
        foreach ($quote->getAllVisibleItems() as $item) {
            $item_option = $item->getOptionByCode('simple_product');
            $cart[] = array(
                'external_id' => $item->getProduct()->getId(),
                'variant_id' => $item_option ? $item_option->getProduct()->getId() : null,
                'quantity' => $item->getQty(),
                'custom_options' => $item->getBuyRequest()->getOptions(),
            );
        }
        return $cart;
    }

    /**
     * Return version 2 cart data
     *
     * Example request:
     *
     * GET https://www.example.com/webinterpret_connector/cart?v=2
     *
     * Example response:
     * {
     *   "magento_version": "1.9.3.8",
     *   "connector_version": "1.6.0.0",
     *   "quote": "base64-encoded-quote-data",
     *   "hash": "hmac-sha-256-hash-on-quote"
     * }
     *
     * Example of quote:
     * {
     *   "entity_id": "680",
     *   "store_id": "1",
     *   "created_at": "2018-11-27 23:50:46",
     *   "updated_at": "2018-11-28 02:41:41",
     *   "converted_at": null,
     *   "is_active": "1",
     *   "is_virtual": "0",
     *   "is_multi_shipping": "0",
     *   "items_count": "2",
     *   "items_qty": "2.0000",
     *   "orig_order_id": "0",
     *   "store_to_base_rate": "1.0000",
     *   "store_to_quote_rate": "1.0000",
     *   "base_currency_code": "EUR",
     *   "store_currency_code": "EUR",
     *   "quote_currency_code": "EUR",
     *   "grand_total": "272.7900",
     *   "base_grand_total": "272.7900",
     *   "checkout_method": null,
     *   "customer_id": null,
     *   "customer_tax_class_id": "10",
     *   "customer_group_id": "0",
     *   "customer_email": null,
     *   "customer_prefix": null,
     *   "customer_firstname": null,
     *   "customer_middlename": null,
     *   "customer_lastname": null,
     *   "customer_suffix": null,
     *   "customer_dob": null,
     *   "customer_note": null,
     *   "customer_note_notify": "1",
     *   "customer_is_guest": "0",
     *   "remote_ip": "172.16.0.46",
     *   "applied_rule_ids": null,
     *   "reserved_order_id": null,
     *   "password_hash": null,
     *   "coupon_code": null,
     *   "global_currency_code": "EUR",
     *   "base_to_global_rate": "1.0000",
     *   "base_to_quote_rate": "1.0000",
     *   "customer_taxvat": null,
     *   "customer_gender": null,
     *   "subtotal": "252.0000",
     *   "base_subtotal": "252.0000",
     *   "subtotal_with_discount": "252.0000",
     *   "base_subtotal_with_discount": "252.0000",
     *   "is_changed": "1",
     *   "trigger_recollect": "0",
     *   "ext_shipping_info": null,
     *   "gift_message_id": null,
     *   "is_persistent": "0",
     *   "customer_balance_amount_used": null,
     *   "base_customer_bal_amount_used": null,
     *   "use_customer_balance": null,
     *   "gift_cards": null,
     *   "gift_cards_amount": null,
     *   "base_gift_cards_amount": null,
     *   "gift_cards_amount_used": null,
     *   "base_gift_cards_amount_used": null,
     *   "gw_id": null,
     *   "gw_allow_gift_receipt": null,
     *   "gw_add_card": null,
     *   "gw_base_price": null,
     *   "gw_price": null,
     *   "gw_items_base_price": null,
     *   "gw_items_price": null,
     *   "gw_card_base_price": null,
     *   "gw_card_price": null,
     *   "gw_base_tax_amount": null,
     *   "gw_tax_amount": null,
     *   "gw_items_base_tax_amount": null,
     *   "gw_items_tax_amount": null,
     *   "gw_card_base_tax_amount": null,
     *   "gw_card_tax_amount": null,
     *   "use_reward_points": null,
     *   "reward_points_balance": null,
     *   "base_reward_currency_amount": null,
     *   "reward_currency_amount": null,
     *   "x_forwarded_for": null,
     *   "wi_items": [
     *     {
     *       "item_id": "2476",
     *       "quote_id": "680",
     *       "created_at": "2018-11-27 23:51:12",
     *       "updated_at": "2018-11-27 23:51:12",
     *       "product_id": "380",
     *       "store_id": "1",
     *       "parent_item_id": null,
     *       "is_virtual": "0",
     *       "sku": "hdb002",
     *       "name": "Shea Enfused Hydrating Body Lotion",
     *       "description": null,
     *       "applied_rule_ids": null,
     *       "additional_data": null,
     *       "free_shipping": "0",
     *       "is_qty_decimal": "0",
     *       "no_discount": "0",
     *       "weight": "1.0000",
     *       "qty": 1,
     *       "price": "28.0000",
     *       "base_price": "28.0000",
     *       "custom_price": null,
     *       "discount_percent": "0.0000",
     *       "discount_amount": "0.0000",
     *       "base_discount_amount": "0.0000",
     *       "tax_percent": "8.2500",
     *       "tax_amount": "2.3100",
     *       "base_tax_amount": "2.3100",
     *       "row_total": "28.0000",
     *       "base_row_total": "28.0000",
     *       "row_total_with_discount": "0.0000",
     *       "row_weight": "1.0000",
     *       "product_type": "simple",
     *       "base_tax_before_discount": null,
     *       "tax_before_discount": null,
     *       "original_custom_price": null,
     *       "redirect_url": null,
     *       "base_cost": null,
     *       "price_incl_tax": "30.3100",
     *       "base_price_incl_tax": "30.3100",
     *       "row_total_incl_tax": "30.3100",
     *       "base_row_total_incl_tax": "30.3100",
     *       "hidden_tax_amount": "0.0000",
     *       "base_hidden_tax_amount": "0.0000",
     *       "gift_message_id": null,
     *       "weee_tax_disposition": "0.0000",
     *       "weee_tax_row_disposition": "0.0000",
     *       "base_weee_tax_disposition": "0.0000",
     *       "base_weee_tax_row_disposition": "0.0000",
     *       "weee_tax_applied": "a:0:{}",
     *       "weee_tax_applied_amount": "0.0000",
     *       "weee_tax_applied_row_amount": "0.0000",
     *       "base_weee_tax_applied_amount": "0.0000",
     *       "base_weee_tax_applied_row_amnt": null,
     *       "event_id": null,
     *       "giftregistry_item_id": null,
     *       "gw_id": null,
     *       "gw_base_price": null,
     *       "gw_price": null,
     *       "gw_base_tax_amount": null,
     *       "gw_tax_amount": null,
     *       "qty_options": [],
     *       "product": [],
     *       "tax_class_id": "2",
     *       "is_recurring": "0",
     *       "has_error": false,
     *       "wi_product": {
     *         "entity_id": "380",
     *         "entity_type_id": "4",
     *         "attribute_set_id": "16",
     *         "type_id": "simple",
     *         "sku": "hdb002",
     *         "has_options": "0",
     *         "required_options": "0",
     *         "created_at": "2013-03-05 05:48:19",
     *         "updated_at": "2013-03-20 16:46:25",
     *         "name": "Shea Enfused Hydrating Body Lotion",
     *         "small_image": "\/h\/d\/hdb002_1.jpg",
     *         "thumbnail": "\/h\/d\/hdb002_1.jpg",
     *         "url_key": "shea-enfused-hydrating-body-lotion",
     *         "url_path": "shea-enfused-hydrating-body-lotion.html",
     *         "msrp_enabled": "2",
     *         "msrp_display_actual_price_type": "4",
     *         "gift_message_available": null,
     *         "status": "1",
     *         "visibility": "4",
     *         "tax_class_id": "2",
     *         "is_recurring": "0",
     *         "weight": "1.0000",
     *         "price": "28.0000",
     *         "special_price": null,
     *         "msrp": null,
     *         "special_from_date": null,
     *         "special_to_date": null,
     *         "is_salable": "1",
     *         "stock_item": [],
     *         "do_not_use_category_id": true,
     *         "request_path": "shea-enfused-hydrating-body-lotion.html",
     *         "tier_price": [],
     *         "is_in_stock": "1",
     *         "store_id": "1",
     *         "customer_group_id": "0",
     *         "final_price": null
     *       },
     *       "wi_order_options": [],
     *       "wi_buy_request": {
     *         "product": "380",
     *         "form_key": "HsNwv6w8Bc8Q8Mal",
     *         "related_product": "",
     *         "qty": 1,
     *         "original_qty": "1"
     *       }
     *     },
     *     {
     *       "item_id": "2477",
     *       "quote_id": "680",
     *       "created_at": "2018-11-28 02:21:20",
     *       "updated_at": "2018-11-28 02:21:20",
     *       "product_id": "423",
     *       "store_id": "1",
     *       "parent_item_id": null,
     *       "is_virtual": "0",
     *       "sku": "wsd007",
     *       "name": "Racer Back Maxi Dress",
     *       "description": null,
     *       "applied_rule_ids": null,
     *       "additional_data": null,
     *       "free_shipping": "0",
     *       "is_qty_decimal": "0",
     *       "no_discount": "0",
     *       "weight": "1.0000",
     *       "qty": 1,
     *       "price": "224.0000",
     *       "base_price": "224.0000",
     *       "custom_price": null,
     *       "discount_percent": "0.0000",
     *       "discount_amount": "0.0000",
     *       "base_discount_amount": "0.0000",
     *       "tax_percent": "8.2500",
     *       "tax_amount": "18.4800",
     *       "base_tax_amount": "18.4800",
     *       "row_total": "224.0000",
     *       "base_row_total": "224.0000",
     *       "row_total_with_discount": "0.0000",
     *       "row_weight": "1.0000",
     *       "product_type": "configurable",
     *       "base_tax_before_discount": null,
     *       "tax_before_discount": null,
     *       "original_custom_price": null,
     *       "redirect_url": null,
     *       "base_cost": null,
     *       "price_incl_tax": "242.4800",
     *       "base_price_incl_tax": "242.4800",
     *       "row_total_incl_tax": "242.4800",
     *       "base_row_total_incl_tax": "242.4800",
     *       "hidden_tax_amount": "0.0000",
     *       "base_hidden_tax_amount": "0.0000",
     *       "gift_message_id": null,
     *       "weee_tax_disposition": "0.0000",
     *       "weee_tax_row_disposition": "0.0000",
     *       "base_weee_tax_disposition": "0.0000",
     *       "base_weee_tax_row_disposition": "0.0000",
     *       "weee_tax_applied": "a:0:{}",
     *       "weee_tax_applied_amount": "0.0000",
     *       "weee_tax_applied_row_amount": "0.0000",
     *       "base_weee_tax_applied_amount": "0.0000",
     *       "base_weee_tax_applied_row_amnt": null,
     *       "event_id": null,
     *       "giftregistry_item_id": null,
     *       "gw_id": null,
     *       "gw_base_price": null,
     *       "gw_price": null,
     *       "gw_base_tax_amount": null,
     *       "gw_tax_amount": null,
     *       "has_children": true,
     *       "qty_options": {
     *         "304": []
     *       },
     *       "product": [],
     *       "tax_class_id": "2",
     *       "is_recurring": null,
     *       "has_error": false,
     *       "wi_product": {
     *         "entity_id": "423",
     *         "entity_type_id": "4",
     *         "attribute_set_id": "13",
     *         "type_id": "configurable",
     *         "sku": "wsd005c",
     *         "has_options": "1",
     *         "required_options": "1",
     *         "created_at": "2013-03-05 06:25:11",
     *         "updated_at": "2013-05-14 14:30:53",
     *         "name": "Racer Back Maxi Dress",
     *         "small_image": "\/w\/s\/wsd005t_2.jpg",
     *         "thumbnail": "\/w\/s\/wsd005t_2.jpg",
     *         "url_key": "racer-back-maxi-dress",
     *         "url_path": "racer-back-maxi-dress-603.html",
     *         "msrp_enabled": "2",
     *         "msrp_display_actual_price_type": "4",
     *         "gift_message_available": null,
     *         "status": "1",
     *         "visibility": "4",
     *         "tax_class_id": "2",
     *         "price": "280.0000",
     *         "special_price": "224.0000",
     *         "msrp": null,
     *         "special_from_date": "2013-05-10 00:00:00",
     *         "special_to_date": null,
     *         "is_salable": "1",
     *         "stock_item": [],
     *         "do_not_use_category_id": true,
     *         "request_path": "racer-back-maxi-dress-603.html",
     *         "tier_price": [],
     *         "is_in_stock": "1",
     *         "store_id": "1",
     *         "customer_group_id": "0",
     *         "final_price": null
     *       },
     *       "wi_order_options": {
     *         "attributes_info": [
     *           {
     *             "label": "Color",
     *             "value": "Purple"
     *           },
     *           {
     *             "label": "Size",
     *             "value": "L"
     *           }
     *         ],
     *         "simple_name": "Racer Back Maxi Dress",
     *         "simple_sku": "wsd007",
     *         "product_calculations": 1,
     *         "shipment_type": 0,
     *         "wi_variant_id": "304",
     *         "wi_variant_weight": "1.0000",
     *         "wi_attributes_merged": [
     *           {
     *             "label": "Color",
     *             "value": "Purple",
     *             "option_id": 92,
     *             "value_id": "18"
     *           },
     *           {
     *             "label": "Size",
     *             "value": "L",
     *             "option_id": 180,
     *             "value_id": "78"
     *           }
     *         ]
     *       },
     *       "wi_buy_request": {
     *         "product": "423",
     *         "form_key": "HsNwv6w8Bc8Q8Mal",
     *         "related_product": "",
     *         "super_attribute": {
     *           "92": "18",
     *           "180": "78"
     *         },
     *         "qty": 1,
     *         "original_qty": "1"
     *       }
     *     }
     *   ]
     * }
     */
    private function buildVersion2Data()
    {
        $quote = Mage::getSingleton('checkout/cart')->getQuote();

        return Mage::helper('glopal_connector')->buildVersion2Data($quote);
    }
}
