<?php

/**
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_ConfigurationController extends Mage_Core_Controller_Front_Action
{
    /**
     * @deprecated
     */
    public function indexAction()
    {
        try {
            Glopal_Connector_Helper_Verifier::verifyRequestWebInterpretSignature();

            if ($_GET['action'] === 'update-option') {
                $this->handleUpdateOptionRequest();
            }
        } catch (Glopal_Connector_Model_SignatureException $e) {
            header('HTTP/1.0 403 Forbidden');
            echo $e->getMessage();
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
        die;
    }

    /**
     * Update given option
     *
     * @deprecated
     */
    private function handleUpdateOptionRequest()
    {
        try {
            if (!isset($_POST['key']) || !isset($_POST['value'])) {
                throw new Exception('Please provide $key and $value to update.');
            }

            $key = filter_var($_POST['key'], FILTER_SANITIZE_STRING);
            $value = filter_var($_POST['value'], FILTER_SANITIZE_STRING);

            if (strpos($key, 'webinterpret_connector/') === 0) {
                $key = substr($key, strlen('webinterpret_connector/'));
            }

            if (preg_match('/[^a-z_]/', $key)) {
                throw new Exception('Invalid characters (only a-z and underscore are allowed)');
            }

            if( $key === 'public_key' ) {
                $value = str_replace('\\n', "\n", $value);
            }

            $key = 'webinterpret_connector/'.$key;

            echo 'Setting key: '.$key.' to value: '.$value.PHP_EOL;

            Mage::getConfig()->saveConfig($key, $value);
            Mage::getConfig()->reinit();

            echo 'Setting successfully updated!'.PHP_EOL;
        } catch (Exception $e) {
            echo 'ERROR: '.$e->getMessage().PHP_EOL;
        }
    }

    /**
     * View configuration action
     */
    public function viewAction()
    {
        try {
            Glopal_Connector_Helper_Verifier::verifyRequestWebInterpretSignature();
            $this->_viewConfig();
        } catch (Glopal_Connector_Model_SignatureException $e) {
            header('HTTP/1.0 403 Forbidden');
            echo $e->getMessage();
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * View configuration
     */
    private function _viewConfig()
    {
        $data = array();
        $fields = $this->getRequest()->getParam('fields');
        
        if( ! empty( $fields ) && is_array( $fields ) ) {
            foreach( $fields as $field ) {

                $path = $field['path'];
                $scope = $field['scope'];
                $scopeId = $field['scope_id'];

                if( $scope == 'websites' ) {
                    $field['value'] = Mage::app()->getWebsite($scopeId)->getConfig( $path );
                } else {
                    $field['value'] = Mage::getStoreConfig( $path, $scopeId );
                }
                $data[]= $field;
            }
        }
        
        $this->getResponse()->clearHeaders()->setHeader( 'Content-type', 'application/json', true );
        $this->getResponse()->setBody( Mage::helper('core')->jsonEncode( $data ) );
    }

    /**
     * Save configuration action
     */
    public function saveAction()
    {
        try {
            Glopal_Connector_Helper_Verifier::verifyRequestWebInterpretSignature();
            $this->_saveConfig();
        } catch (Glopal_Connector_Model_SignatureException $e) {
            header('HTTP/1.0 403 Forbidden');
            echo $e->getMessage();
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * Save configuration
     */
    private function _saveConfig()
    {
        $fields = $this->getRequest()->getPost('fields');

        if( ! empty( $fields ) && is_array( $fields ) ) {

            foreach( $fields as $field ) {

                $path = $field['path'];
                $scope = $field['scope'];
                $scopeId = $field['scope_id'];
                $value = $field['value'];

                $this->validateScope($scope, $scopeId);

                if ($value === null) {
                    Mage::getConfig()->deleteConfig( $path, $scope, $scopeId );
                } else {
                    Mage::getConfig()->saveConfig($path, $value, $scope, $scopeId);
                }
            }

            Mage::getConfig()->reinit();
        }

        $this->getResponse()->clearHeaders()->setHeader( 'Content-type', 'application/json', true );
        $this->getResponse()->setBody( Mage::helper('core')->jsonEncode( ['status' => 'ok'] ) );
    }

    /**
     * Validate scope and scope id.
     *
     * @param $scope
     * @param $scopeId
     *
     * @return void
     */
    private function validateScope($scope, $scopeId)
    {
        if (empty($scopeId)) {
            return;
        }

        if ($scope === 'websites' || $scope === 'website') {
            $websiteIds = [];
            foreach (Mage::app()->getWebsites() as $website) {
                $websiteIds[] = $website->getId();
            }

            if (!in_array($scopeId, $websiteIds)) {
                throw new Mage_Core_Exception(__(sprintf('Website with given ID: %s doesn\'t exists', $scopeId)));
            }
        }

        if ($scope === 'stores' || $scope === 'store') {
            $storeIds = [];
            foreach (Mage::app()->getStores() as $store) {
                $storeIds[] = $store->getId();
            }

            if (!in_array($scopeId, $storeIds)) {
                throw new Mage_Core_Exception(__(sprintf('Store with given ID: %s doesn\'t exists', $scopeId)));
            }
        }
    }
}

