<?php

require_once('AbstractController.php');

class Glopal_Connector_DispatchController extends Glopal_Connector_AbstractController
{
    const ALLOWED_OBJECTS = [
        'order' => true
    ];
    const DEFAULT_OBJECT_TYPE = 'order';

    /**
     * {@inheritdoc}
     */
    protected function process()
    {
        if (!$this->getRequest()->isPost()) {
            throw new InvalidArgumentException('Wrong method used');
        }
        $objectType = $this->getRequest()->getParam('object_type', self::DEFAULT_OBJECT_TYPE);
        if (!isset(self::ALLOWED_OBJECTS[$objectType]) || !self::ALLOWED_OBJECTS[$objectType]) {
            throw new InvalidArgumentException('Wrong Object Type Provided');
        }
        $entityId = $this->getRequest()->getParam('entity_id', null);
        $eventName = $this->getRequest()->getParam('event_name', null);
        if (empty($entityId)) {
            throw new InvalidArgumentException('Invalid entity id provided');
        }
        if (empty($eventName)) {
            throw new InvalidArgumentException('Invalid event name provided');
        }
        $action = 'process' . ucfirst($objectType);
        if (!method_exists($this, $action)) {
            throw new BadFunctionCallException('Internal Exception, please contat administrator');
        }
        $dispatchKey = $this->getRequest()->getParam('dispatchKey', null);
        if (empty($dispatchKey)) {
            return $this->$action($eventName, $entityId);
        }

        return $this->$action($eventName, $entityId, $dispatchKey);
    }
    
    /**
     * @param string $eventName
     * @param int $entityId
     * @param string $dispatchKey
     * @throws InvalidArgumentException if the order is not found
     */
    protected function processOrder($eventName, $entityId, $dispatchKey = 'order')
    {
        $order = Mage::getModel('sales/order')->load($entityId);
        if (null == $order->getId()) {
            throw new InvalidArgumentException('Entity searched was not found');
        }
        $timesBefore = $this->countObserverCalls(array_keys(Varien_Profiler::getTimers()));
        Varien_Profiler::enable();
        Mage::dispatchEvent($eventName, [$dispatchKey => $order]);
        Varien_Profiler::disable();
        $timesAfter = $this->countObserverCalls(array_keys(Varien_Profiler::getTimers()));
        return ['observersCalled' => $timesAfter - $timesBefore];
    }

    /**
     * @param array $list
     * @return int
     */
    private function countObserverCalls($list) {
        return count(array_filter($list, function($el){ 
            return 0 === strpos($el, 'OBSERVER: ');
        }));
    }
}
