<?php

require_once('AbstractController.php');

class Glopal_Connector_InvoiceController extends Glopal_Connector_AbstractController
{
    const INVOICE_COMMENT_CREATED_BY_GLOPAL = '--INVOICE WAS CREATED BY GLOPAL API--';

    /**
     * @return array
     * @throws InvalidArgumentException if we can't find the order in the database or the order_id param was empty
     */
    protected function listInvoices()
    {
        $orderId = $this->getRequest()->getParam('order_id', null);
        if (empty($orderId)) {
            throw new InvalidArgumentException('Order Id was not provided');
        }
        $order = Mage::getModel('sales/order')->load($orderId);
        if (null == $order->getId()) {
             throw new InvalidArgumentException('Wrong order Id was provided');
        }
        if (!$order->hasInvoices()) {
            return [];
        }
        $invoices = array_values($order->getInvoiceCollection()->getItems());
        return array_map(function($invoice){
           $comments =  $invoice->getCommentsCollection()->getItems();
           $createdByGlopal = array_filter($comments, function($comment) {
               return $comment->getComment() == Glopal_Connector_InvoiceController::INVOICE_COMMENT_CREATED_BY_GLOPAL;
           });
           return [
               'invoice_id'    => $invoice->getId(),
               'grand_total'   => $invoice->getGrandTotal(),
               'total_qty'     => $invoice->getTotalQty(),
               'increment_id'  => $invoice->getIncrementId(),
               'createdByGlopal' => (bool) count($createdByGlopal)
           ];
        }, $invoices);
    }

    /**
     * @return array
     * @throws InvalidArgumentException if we can't find the order in the database or the order_id param was empty
     * @throws OutOfRangeException in case the order was already invoiced or it can't be invoiced
     */
    protected function createInvoice() 
    {
        $orderId = $this->getRequest()->getParam('order_id', null);
        if (empty($orderId)) {
            throw new InvalidArgumentException('Order Id was not provided');
        }
        $order = Mage::getModel('sales/order')->load($orderId);
        if (null == $order->getId()) {
             throw new InvalidArgumentException('Wrong order Id was provided');
        }
        if ($order->hasInvoices()) {
            throw new OutOfRangeException('The order was already invoiced');
        }
        if (!$order->canInvoice()) {
            throw new OutOfRangeException('The order can not be invoiced');
        }
        
        $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();
        $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_OFFLINE);
        $invoice->addComment(
            self::INVOICE_COMMENT_CREATED_BY_GLOPAL
        );
        $invoice->register();
        $invoice->getOrder()->setIsInProcess(true);
        $transactionSave = Mage::getModel('core/resource_transaction')
            ->addObject($invoice)
            ->addObject($invoice->getOrder());
        $transactionSave->save();
        
        return [
            'invoice_id'    => $invoice->getId(),
            'grand_total'   => $invoice->getGrandTotal(),
            'total_qty'     => $invoice->getTotalQty(),
            'increment_id'  => $invoice->getIncrementId(),
            'createdByGlopal' => true
        ];
    }

    /**
     * @return array
     * @throws InvalidArgumentException if a different request method type is used
     */
    protected function process()
    {
        if ($this->getRequest()->isGet()) {
            return $this->listInvoices();
        }
        if ($this->getRequest()->isPost()) {
            return $this->createInvoice();
        }
        throw new InvalidArgumentException('Wrong method used');
    }
}
